#!/usr/bin/env python3
"""
Honeypot Defender
Creates decoy services to detect and trap attackers.
Parameters:
    - ports: Comma-separated list of ports to monitor
    - mode: Operation mode (detect, trap, aggressive)
Requires: admin/root privileges
"""
import socket
import threading
import logging
from datetime import datetime
import json
from pathlib import Path
import queue
import time
import subprocess
import platform
import psutil
import random

class HoneypotDefender:
    def __init__(self, ports, mode='detect', response_delay=1):
        self.ports = ports
        self.mode = mode
        self.response_delay = response_delay
        self.is_windows = platform.system().lower() == 'windows'
        self.attackers = set()
        self.connections = queue.Queue()
        self.stop_event = threading.Event()
        self.setup_logging()
        
        # Decoy service banners
        self.service_banners = {
            21: "220 FTP server (vsftpd 2.3.4)\r\n",
            22: "SSH-2.0-OpenSSH_4.7p1\r\n",
            23: "Welcome to Microsoft Telnet Service\r\n",
            25: "220 mail.example.com ESMTP Postfix\r\n",
            80: "HTTP/1.1 200 OK\r\nServer: Apache/2.2.14\r\n\r\n",
            110: "+OK POP3 server ready\r\n",
            143: "* OK IMAP server ready\r\n",
            443: "HTTP/1.1 200 OK\r\nServer: nginx/1.14.0\r\n\r\n",
            3306: "5.5.5-10.1.37-MariaDB\n",
            5432: "PostgreSQL 9.6.10\n"
        }
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("honeypot_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "honeypot.log"),
                logging.StreamHandler()
            ]
        )
        
    def get_service_banner(self, port):
        """Get appropriate banner for service"""
        return self.service_banners.get(port, f"220 Service Ready\r\n")
        
    def handle_connection(self, client_socket, address, port):
        """Handle incoming connection"""
        try:
            # Log connection attempt
            self.connections.put({
                'timestamp': datetime.now().isoformat(),
                'ip': address[0],
                'port': port,
                'type': 'connection'
            })
            
            if self.mode in ['trap', 'aggressive']:
                # Add attacker to blocked list
                self.attackers.add(address[0])
                
                # Simulate vulnerable service
                time.sleep(self.response_delay)  # Add delay to seem more realistic
                banner = self.get_service_banner(port)
                client_socket.send(banner.encode())
                
                # In aggressive mode, try to gather more information
                if self.mode == 'aggressive':
                    data = client_socket.recv(1024)
                    if data:
                        self.connections.put({
                            'timestamp': datetime.now().isoformat(),
                            'ip': address[0],
                            'port': port,
                            'type': 'data',
                            'data': data.decode(errors='ignore')
                        })
                        
                # Keep connection open longer in trap mode
                time.sleep(5)
                
        except Exception as e:
            logging.error(f"Error handling connection from {address[0]}: {str(e)}")
        finally:
            client_socket.close()
            
    def start_honeypot(self, port):
        """Start honeypot on specified port"""
        try:
            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            server.bind(('0.0.0.0', port))
            server.listen(5)
            server.settimeout(1)  # Allow checking stop_event
            
            logging.info(f"Honeypot listening on port {port}")
            
            while not self.stop_event.is_set():
                try:
                    client, address = server.accept()
                    threading.Thread(
                        target=self.handle_connection,
                        args=(client, address, port)
                    ).start()
                except socket.timeout:
                    continue
                except Exception as e:
                    logging.error(f"Error accepting connection on port {port}: {str(e)}")
                    
        except Exception as e:
            logging.error(f"Error starting honeypot on port {port}: {str(e)}")
        finally:
            server.close()
            
    def block_attacker(self, ip):
        """Block attacking IP address"""
        try:
            if self.is_windows:
                subprocess.run([
                    'netsh', 'advfirewall', 'firewall', 'add', 'rule',
                    f'name="Honeypot Block {ip}"',
                    'dir=in', 'action=block',
                    f'remoteip={ip}'
                ], check=True)
            else:
                subprocess.run([
                    'iptables', '-A', 'INPUT', '-s', ip, '-j', 'DROP'
                ], check=True)
                
            logging.info(f"Blocked attacker IP: {ip}")
            
        except subprocess.CalledProcessError as e:
            logging.error(f"Error blocking IP {ip}: {str(e)}")
            
    def monitor_connections(self):
        """Monitor and process connection attempts"""
        while not self.stop_event.is_set():
            try:
                conn = self.connections.get(timeout=1)
                logging.warning(
                    f"Connection attempt from {conn['ip']} "
                    f"on port {conn['port']}"
                )
                
                if conn['type'] == 'data':
                    logging.warning(
                        f"Received data from {conn['ip']}: "
                        f"{conn['data'][:100]}..."
                    )
                    
                # Block attackers in trap or aggressive mode
                if self.mode in ['trap', 'aggressive'] and conn['ip'] in self.attackers:
                    self.block_attacker(conn['ip'])
                    
            except queue.Empty:
                continue
                
    def create_fake_services(self):
        """Create convincing fake services"""
        services = []
        
        # Create fake process names
        fake_processes = {
            21: "vsftpd",
            22: "sshd",
            23: "telnetd",
            25: "postfix",
            80: "apache2",
            443: "nginx",
            3306: "mysqld",
            5432: "postgres"
        }
        
        for port in self.ports:
            if port in fake_processes:
                service_name = fake_processes[port]
                # TODO: Create fake process entries
                services.append({
                    'port': port,
                    'service': service_name
                })
                
        return services
        
    def deploy(self):
        """Deploy honeypot defender"""
        logging.info("Starting Honeypot Defender")
        logging.info(f"Mode: {self.mode}")
        logging.info(f"Monitoring ports: {', '.join(map(str, self.ports))}")
        
        # Create fake services
        fake_services = self.create_fake_services()
        for service in fake_services:
            logging.info(f"Created fake service: {service['service']} on port {service['port']}")
            
        # Start honeypots for each port
        honeypots = []
        for port in self.ports:
            thread = threading.Thread(target=self.start_honeypot, args=(port,))
            thread.start()
            honeypots.append(thread)
            
        # Start connection monitor
        monitor = threading.Thread(target=self.monitor_connections)
        monitor.start()
        
        try:
            while True:
                # Print status
                attacker_count = len(self.attackers)
                print(f"\rActive attackers: {attacker_count}", end="")
                time.sleep(1)
                
        except KeyboardInterrupt:
            logging.info("\nStopping Honeypot Defender...")
            self.stop_event.set()
            
            # Wait for threads to finish
            for thread in honeypots:
                thread.join()
            monitor.join()
            
            # Save final state
            self.save_state()
            
    def save_state(self):
        """Save honeypot state and statistics"""
        state = {
            'timestamp': datetime.now().isoformat(),
            'mode': self.mode,
            'ports': self.ports,
            'attackers': list(self.attackers)
        }
        
        # Get all connection attempts
        connections = []
        while not self.connections.empty():
            connections.append(self.connections.get())
            
        state['connections'] = connections
        
        # Save to file
        state_file = Path("honeypot_logs") / f"state_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(state_file, 'w') as f:
            json.dump(state, f, indent=2)
            
        logging.info(f"State saved to {state_file}")
        
        # Print summary
        print("\nHoneypot Summary")
        print("=" * 50)
        print(f"Total Attackers: {len(state['attackers'])}")
        print(f"Total Connections: {len(state['connections'])}")
        print("\nTop Attacking IPs:")
        ip_counts = {}
        for conn in state['connections']:
            ip_counts[conn['ip']] = ip_counts.get(conn['ip'], 0) + 1
        for ip, count in sorted(ip_counts.items(), key=lambda x: x[1], reverse=True)[:10]:
            print(f"{ip}: {count} connections")

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Honeypot Defender")
    parser.add_argument("ports", help="Comma-separated list of ports to monitor")
    parser.add_argument("--mode", choices=['detect', 'trap', 'aggressive'],
                      default='detect', help="Operation mode")
    parser.add_argument("--delay", type=float, default=1.0,
                      help="Response delay in seconds")
    args = parser.parse_args()
    
    ports = [int(p) for p in args.ports.split(',')]
    
    defender = HoneypotDefender(ports, args.mode, args.delay)
    defender.deploy()
